@@ -98,7 +98,7 @@ void tsclient_applet_create (PanelApplet *applet)
 
   g_data->applet = applet;
 
-  g_data->tooltips = gtk_tooltips_new();
+  g_data->tooltips = gtk_tooltips_new ();
   gtk_tooltips_set_tip (g_data->tooltips,
           GTK_WIDGET(g_data->applet), 
           _("Terminal Server Client Applet"), NULL);
@@ -148,22 +148,68 @@ void tsclient_applet_create (PanelApplet *applet)
   gtk_widget_show_all (GTK_WIDGET (g_data->applet));
 }
 
+void
+create_menu (gpointer key, gpointer value, gpointer menu)
+{
+  GtkMenu* submenu = NULL;
+  GtkMenuItem *subitem =NULL;
+  GtkMenuItem* itemfound = NULL;
+  
+  GList* list = NULL;
+  gchar* label_text = NULL;
+  gchar *name = g_strdup (key);
+  gchar *value_c = g_strdup (value);
+  
+  GtkMenuItem *mi = (GtkMenuItem *) gtk_menu_item_new_with_label (name);
+	
+  list = gtk_container_get_children (GTK_CONTAINER(menu));
+  itemfound = NULL;  
+  for ( ; (list != NULL) && (itemfound == NULL) ; list = g_list_next (list)) {
+    if (GTK_IS_MENU_ITEM(list->data)) {
+      if (GTK_IS_LABEL(GTK_BIN(list->data)->child)) {
+	      label_text = gtk_label_get_text (GTK_LABEL(GTK_BIN(list->data)->child));
+	      if (g_ascii_strcasecmp (label_text, value_c) == 0)
+	        itemfound = GTK_MENU_ITEM(list->data);
+      }
+    }
+  }
+
+  if (g_ascii_strcasecmp (value_c, "-1") != 0){
+	  
+  	g_signal_connect (GTK_WIDGET (mi), "activate",GTK_SIGNAL_FUNC (applet_menu_item),
+	  	g_build_path ("/", value_c, name, NULL));
+	  
+    if (itemfound != NULL) {
+	    submenu = gtk_menu_item_get_submenu (itemfound);
+	    subitem = itemfound;
+    } else {
+	    submenu = GTK_MENU (gtk_menu_new ());
+	    subitem = (GtkMenuItem *) gtk_menu_item_new_with_label (value_c);
+	    gtk_menu_shell_append ((GtkMenuShell *) menu, GTK_WIDGET (subitem));
+	    gtk_widget_show (GTK_WIDGET (subitem));
+    }
+		
+    gtk_menu_item_set_submenu (subitem, GTK_WIDGET (submenu));
+    gtk_menu_shell_append ((GtkMenuShell *) submenu, GTK_WIDGET (mi));
+    
+    gtk_widget_show (GTK_WIDGET (mi));  
+  } else {
+  	g_signal_connect (GTK_WIDGET (mi), "activate",GTK_SIGNAL_FUNC (applet_menu_item), name);
+    if (itemfound == NULL) {
+      gtk_menu_shell_append ((GtkMenuShell *) menu, GTK_WIDGET (mi));
+      gtk_widget_show (GTK_WIDGET (mi));
+    }
+  }
+}
+
 
 gboolean applet_popup_show (GtkWidget  *widget,
                             GdkEvent   *event,
                             AppletData *data)
 {
-  GtkRequisition req;
-  gint x, y;
-  gint width, height;
-  guint n = 0;
-  GdkGrabStatus pointer, keyboard;
-  GtkWidget *dialog;
-  GSList    *lptr = NULL;
   GtkMenu   *menu = NULL;
   GtkMenuItem *mi = NULL;
-  gchar *name = NULL;
-  gint cnt, ret;
+  GHashTable* hash=NULL;
   
   g_return_val_if_fail (widget != NULL, FALSE) ;
   g_return_val_if_fail (event != NULL, FALSE) ;
@@ -173,26 +219,28 @@ gboolean applet_popup_show (GtkWidget  *widget,
   case GDK_BUTTON_RELEASE :
     if (event->button.button == 1) { 
       /*user pressed the left mouse button*/
-      ret = rdp_files_to_list (&lptr);
-      cnt = 0;
-      menu = GTK_MENU (gtk_menu_new ());
-      while (lptr) {
-        name = g_strdup (lptr->data);
-        mi = (GtkMenuItem*) gtk_menu_item_new_with_label (name);
-        g_signal_connect (GTK_WIDGET (mi), "activate", GTK_SIGNAL_FUNC (applet_menu_item), name);
-        gtk_widget_ref (GTK_WIDGET (mi));
-        gtk_menu_shell_append ((GtkMenuShell *)menu, GTK_WIDGET (mi));
-        gtk_widget_show (GTK_WIDGET (mi));
 
-        /* move forward */
-        lptr = lptr->next;
-        cnt++;
-      }
-      gtk_menu_popup (menu, NULL, NULL, 
-          NULL, data, event->button.button, gtk_get_current_event_time ());
-      g_slist_free (lptr);
-      data->popup = menu;
-      g_object_ref (G_OBJECT (data->popup));
+	  hash = rdp_files_to_hash ();
+	  menu = GTK_MENU (gtk_menu_new ());
+
+	  g_hash_table_foreach (hash, create_menu, menu);
+
+	  mi = (GtkMenuItem *) gtk_separator_menu_item_new ();
+	  gtk_menu_shell_append ((GtkMenuShell *) menu, GTK_WIDGET (mi));
+	  gtk_widget_show (GTK_WIDGET (mi));
+
+	  mi = (GtkMenuItem *) gtk_menu_item_new_with_label (_("New Connection..."));
+	  g_signal_connect (GTK_WIDGET(mi), "activate", GTK_SIGNAL_FUNC(applet_launch_tsclient), NULL);
+	  gtk_menu_shell_append ((GtkMenuShell *) menu, GTK_WIDGET (mi));
+	  gtk_widget_show (GTK_WIDGET (mi));
+
+      gtk_menu_popup (menu, NULL, NULL, NULL, data,
+			  event->button.button,
+			  gtk_get_current_event_time ());
+	  
+	  data->popup = menu;
+	  g_hash_table_destroy (hash);
+	  
       return TRUE;
     }
     break ;
@@ -215,13 +263,6 @@ void applet_popup_hide (AppletData *data, gboolean revert)
   if (data->popup) {
     gdk_pointer_ungrab (GDK_CURRENT_TIME);
     gdk_keyboard_ungrab (GDK_CURRENT_TIME);
-
-    /* Ref the adjustment or it will get destroyed
-       when the scale goes away. */
-    //g_object_ref(G_OBJECT(data->adj));
-
-    //gtk_widget_destroy (GTK_WIDGET(data->popup));
-    //data->popup = NULL;
   }
 }
 
@@ -237,7 +278,7 @@ void applet_menu_item (GtkMenuItem *menuitem, gpointer user_data)
   rdp_file *rdp = NULL;
   GtkWidget *dialog;
   // build path to file
-  file_name = g_build_path ((const gchar *)"/", g_get_home_dir(), ".tsclient", g_strdup (user_data), NULL);
+  file_name = g_build_path ((const gchar *)"/", g_get_home_dir (), ".tsclient", g_strdup (user_data), NULL);
 
   // check for file in ~/
   if (g_file_test (file_name, G_FILE_TEST_EXISTS)) {
@@ -285,7 +326,7 @@ void applet_destroy (GtkWidget *widget, AppletData *data)
     gtk_timeout_remove (data->timeout);
     data->timeout = 0;
   }
-  g_free(data);
+  g_free (data);
 }
 
 
@@ -338,7 +379,7 @@ void applet_launch_tsclient (BonoboUIComponent *uic, AppletData *data, const gch
 
   if (!run_tsclient_cmd) return;
 
-  c_argv[c_argc++] = strdup(run_tsclient_cmd);
+  c_argv[c_argc++] = strdup (run_tsclient_cmd);
   c_argv[c_argc++] = NULL;
 
   // complete events in gtk queue
@@ -383,4 +424,3 @@ void add_atk_namedesc (GtkWidget *widget, const gchar *name, const gchar *desc)
   atk_object_set_name (atk_obj, name);
   atk_object_set_description (atk_obj, desc);
 }
-
