@@ -39,18 +39,25 @@ int
 create_frmConnect (void)
 {
   gint i = 0;
-  gchar *langlist[]={    "", "be", "bg", "br", "cf", "ch", 
-    "cz", "dk", "dv", "et", "fr", "gk", 
-    "gr", "hu", "is", "it", "jp", "ko", 
-    "la", "nl", "no", "pl", "po", "ro",
-    "ru", "sf", "sl", "sp", "su", "sv",
-    "tr", "uk", "us", "usl", "usr", "yu", 
+  gchar *langlist[]={    "", "ar", "be", 
+    "bg", "br", "cf", "ch", "common",
+    "cz", "da", "de", "de-ch", "dk", "dv", 
+    "en-gb", "en-us", "es", "et", "fi", 
+    "fo", "fr", "fr-be", "fr-ca", "fr-ch", 
+    "gk", "gr", "hr", "hu", "is", "it", 
+    "ja", "ko", "la", "lt", "lv", "mk", 
+    "nl", "no", "pl", "po", "pt", "pt-br",
+    "ro", "ru", "sf", "sl", "sp", "su", 
+    "sv", "th", "tr", "uk", "us", "usl", 
+    "usr", "yu",
     NULL};
   gchar *size_items[] = {
-    _("640 x 480 pixels"),    _("800 x 600 pixels"),
+    _("640 x 480 pixels"),
+    _("800 x 600 pixels"),
     _("1024 x 768 pixels"),
     _("1152 x 864 pixels"),
     _("1280 x 960 pixels"),
+    _("1400 x 1050 pixels"),
 	  NULL
   };
   gchar *color_items[] = {
@@ -201,6 +208,7 @@ create_frmConnect (void)
     GtkWidget *chkNoMotionEvents;
     GtkWidget *chkEnableWMKeys;
     GtkWidget *chkHideWMDecorations;
+    GtkWidget *chkAttachToConsole;
 
   // Complete Button Box Widgets
   GtkWidget *hbbAppOps;
@@ -914,6 +922,10 @@ create_frmConnect (void)
   gtk_widget_set_name (chkHideWMDecorations, "chkHideWMDecorations");
   gtk_box_pack_start (GTK_BOX (vbxExpChecks), chkHideWMDecorations, FALSE, FALSE, 0);
 
+  chkAttachToConsole = gtk_check_button_new_with_mnemonic (_("Attach to console"));
+  gtk_widget_set_name (chkAttachToConsole, "chkAttachToConsole");
+  gtk_box_pack_start (GTK_BOX (vbxExpChecks), chkAttachToConsole, FALSE, FALSE, 0);
+
 
   lblPerformanceOptions = gtk_label_new (_("Options available for optimizing performance"));
   gtk_widget_set_name (lblPerformanceOptions, "lblPerformanceOptions");
@@ -1218,6 +1230,7 @@ create_frmConnect (void)
     HOOKUP_OBJECT (frmConnect, chkNoMotionEvents, "chkNoMotionEvents");
     HOOKUP_OBJECT (frmConnect, chkEnableWMKeys, "chkEnableWMKeys");
     HOOKUP_OBJECT (frmConnect, chkHideWMDecorations, "chkHideWMDecorations");
+    HOOKUP_OBJECT (frmConnect, chkAttachToConsole, "chkAttachToConsole");
 
   // Complete Button Box Widgets
   HOOKUP_OBJECT (frmConnect, hbbAppOps, "hbbAppOps");
@@ -1274,6 +1287,8 @@ on_btnConnect_clicked                  (GtkButton       *button,
 {
   GtkWidget *main_window;
   rdp_file *rdp = NULL;
+  gchar *home = tsc_home_path ();
+  gchar *filename = g_build_path ("/", home, "last.tsc");
 
   #ifdef TSCLIENT_DEBUG
   printf ("on_btnConnect_clicked\n");
@@ -1285,7 +1300,7 @@ on_btnConnect_clicked                  (GtkButton       *button,
   rdp_file_init (rdp);
   rdp_file_get_screen (rdp, main_window);
   // save the damn thing as last
-  rdp_file_save (rdp, g_build_path ("/", tsc_home_path(), "last.tsc", NULL));
+  rdp_file_save (rdp, filename);
 
   gtk_widget_hide (main_window);
   gtk_widget_destroy (main_window);
@@ -1297,6 +1312,9 @@ on_btnConnect_clicked                  (GtkButton       *button,
   create_frmConnect ();
   rdp_file_set_screen (rdp, gConnect);
   g_free (rdp);
+
+  g_free (filename);
+  g_free (home);
 }
 
 
@@ -1306,6 +1324,8 @@ on_btnQuit_clicked                     (GtkButton       *button,
 {
   GtkWidget *main_window;
   rdp_file *rdp_last = NULL;
+  gchar *home = tsc_home_path ();
+  gchar *filename = g_build_path ("/", home, "last.tsc");
 
   #ifdef TSCLIENT_DEBUG
   printf ("on_btnQuit_clicked\n");
@@ -1318,9 +1338,12 @@ on_btnQuit_clicked                     (GtkButton       *button,
   rdp_last = g_new0 (rdp_file, 1);
   rdp_file_init (rdp_last);
   rdp_file_get_screen (rdp_last, main_window);
-  rdp_file_save (rdp_last, g_build_path ("/", tsc_home_path(), "last.tsc", NULL));
+  rdp_file_save (rdp_last, filename);
   g_free (rdp_last);
 
+  g_free (filename);
+  g_free (home);
+
   gtk_main_quit();
 }
 
@@ -1402,35 +1425,52 @@ on_btnSaveAs_clicked                   (GtkButton       *button,
   GtkWidget *main_window;
   rdp_file *rdp = NULL;
 
-  save_selector = gtk_file_selection_new ("Select File");
-  gtk_widget_set_name (save_selector, "save_selector");
-  gtk_container_set_border_width (GTK_CONTAINER (save_selector), 12);
-  gtk_window_set_position (GTK_WINDOW (save_selector), GTK_WIN_POS_CENTER);
-  gtk_window_set_modal (GTK_WINDOW (save_selector), TRUE);
-  gtk_window_set_destroy_with_parent (GTK_WINDOW (save_selector), TRUE);
-  gtk_window_set_title ((GtkWindow*) save_selector, _("Input a filename to save as..."));
-  gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (save_selector));
-  gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (save_selector), FALSE);
-  gtk_window_set_icon (GTK_WINDOW (save_selector), create_pixbuf ("tsclient.png"));
-  gtk_widget_grab_default (GTK_FILE_SELECTION (save_selector)->ok_button);
-  gtk_file_selection_set_filename (GTK_FILE_SELECTION (save_selector), g_build_path ("/", tsc_home_path(), "/", NULL));
+  main_window = lookup_widget((GtkWidget *)button, "frmConnect");
 
-  if (gtk_dialog_run (GTK_DIALOG (save_selector)) == GTK_RESPONSE_OK) {
-    gchar *filename = (gchar*)gtk_file_selection_get_filename (GTK_FILE_SELECTION (save_selector));
+  save_selector = gtk_file_chooser_dialog_new (_("Input a filename to save as..."), 
+    GTK_WINDOW(main_window), GTK_FILE_CHOOSER_ACTION_SAVE,
+	GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+	GTK_STOCK_OK, GTK_RESPONSE_OK,
+	NULL);
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(save_selector), tsc_home_path ());
+  
+  if (gtk_dialog_run (GTK_DIALOG(save_selector)) == GTK_RESPONSE_OK) {
+    gchar *filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(save_selector));
     if (filename) {
+      GtkWidget *mi;
+	  char *home = tsc_home_path ();
+      // Add our extension if needed
+	  char *lastdelim = g_strrstr (filename, "/");
+      if (lastdelim && g_strrstr (lastdelim + 1, ".") == NULL) {
+	    char *tmp = g_strconcat (filename, ".rdp", NULL);
+		g_free (filename);
+		filename = tmp;
+      }
       #ifdef TSCLIENT_DEBUG
       printf ("filename: %s\n", filename);
       #endif
-      main_window = lookup_widget((GtkWidget*)button, "frmConnect");
       // save rdp file
       rdp = g_new0 (rdp_file, 1);
       rdp_file_init (rdp);
       rdp_file_get_screen (rdp, main_window);
       rdp_file_save (rdp, filename);
       g_free (rdp);
+      // Add to quick pick list
+	  lastdelim = g_strrstr (filename, tsc_home_path ());
+	  if (lastdelim) {
+        GtkWidget *opt = lookup_widget (main_window, "optProfileLauncher");
+ 	    lastdelim += strlen (home) + 1;
+  	    mi = gtk_menu_item_new_with_mnemonic (lastdelim);
+        g_signal_connect (G_OBJECT(mi), "activate", G_CALLBACK(tsc_quick_pick_activate), g_strdup (lastdelim));
+        gtk_container_add (GTK_CONTAINER(GTK_OPTION_MENU(opt)->menu), mi);
+        gtk_widget_show (mi);
+	  }
+	  // Try to avoid *some* memory leaks
+      g_free (filename);
+	  g_free (home);
     }
   }
-  gtk_widget_destroy (GTK_WIDGET (save_selector));
+  gtk_widget_destroy (GTK_WIDGET(save_selector));
   return;
 }
 
@@ -1440,41 +1480,44 @@ on_btnOpen_clicked                     (GtkButton       *button,
                                         gpointer         user_data)
 {
   GtkWidget *open_selector = NULL;
+  GtkFileFilter *filter;
   GtkWidget *main_window;
   rdp_file *rdp = NULL;
 
-  open_selector = gtk_file_selection_new ("Select File");
-  gtk_widget_set_name (open_selector, "open_selector");
-  gtk_container_set_border_width (GTK_CONTAINER (open_selector), 12);
-  gtk_window_set_position (GTK_WINDOW (open_selector), GTK_WIN_POS_CENTER);
-  gtk_window_set_modal (GTK_WINDOW (open_selector), TRUE);
-  gtk_window_set_destroy_with_parent (GTK_WINDOW (open_selector), TRUE);
-  gtk_window_set_title ((GtkWindow*) open_selector, _("Choose an RDP file to open..."));
-  gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (open_selector));
-  gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (open_selector), FALSE);
-  gtk_window_set_icon (GTK_WINDOW (open_selector), create_pixbuf ("tsclient.png"));
-  gtk_widget_grab_default (GTK_FILE_SELECTION (open_selector)->ok_button);
-  gtk_file_selection_set_filename (GTK_FILE_SELECTION (open_selector), g_build_path ("/", tsc_home_path(), "/", NULL));
+  main_window = lookup_widget((GtkWidget *)button, "frmConnect");
 
-  if (gtk_dialog_run (GTK_DIALOG (open_selector)) == GTK_RESPONSE_OK) {
-    gchar *filename = (gchar*)gtk_file_selection_get_filename (GTK_FILE_SELECTION (open_selector));
+  open_selector = gtk_file_chooser_dialog_new (_("Choose an RDP file to open..."), 
+    GTK_WINDOW(main_window), GTK_FILE_CHOOSER_ACTION_OPEN,
+	GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+	GTK_STOCK_OK, GTK_RESPONSE_OK,
+	NULL);
+  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(open_selector), tsc_home_path ());
+
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name (filter, "RDP Files");
+  gtk_file_filter_add_pattern (filter, "*.rdp");
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(open_selector), filter);
+  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER(open_selector), filter);
+  
+  if (gtk_dialog_run (GTK_DIALOG(open_selector)) == GTK_RESPONSE_OK) {
+    gchar *filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(open_selector));
     if (filename) {
       #ifdef TSCLIENT_DEBUG
       printf ("filename: %s\n", filename);
       #endif
-      main_window = lookup_widget((GtkWidget*)button, "frmConnect");
-      // check for file in ~/
+      // check if the file exists
       if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
         // load if exists
         rdp = g_new0 (rdp_file, 1);
         rdp_file_init (rdp);
         rdp_file_load (rdp, filename);
         rdp_file_set_screen (rdp, main_window);
+        g_free (rdp);
+        g_free (filename);
       }
-      g_free (rdp);
     }
   }
-  gtk_widget_destroy (GTK_WIDGET (open_selector));
+  gtk_widget_destroy (GTK_WIDGET(open_selector));
   return;
 }
 
@@ -1485,6 +1528,7 @@ on_btnProtoFile_clicked                (GtkButton       *button,
 {
   GtkWidget *open_selector = NULL;
   GtkWidget *main_window, *proto_file;
+  gchar *home = tsc_home_path ();
   guint pos = 0;
 
   open_selector = gtk_file_selection_new ("Select File");
@@ -1498,10 +1542,10 @@ on_btnProtoFile_clicked                (GtkButton       *button,
   gtk_file_selection_set_select_multiple (GTK_FILE_SELECTION (open_selector), FALSE);
   gtk_window_set_icon (GTK_WINDOW (open_selector), create_pixbuf ("tsclient.png"));
   gtk_widget_grab_default (GTK_FILE_SELECTION (open_selector)->ok_button);
-  gtk_file_selection_set_filename (GTK_FILE_SELECTION (open_selector), g_build_path ("/", tsc_home_path(), "/", NULL));
+  gtk_file_selection_set_filename (GTK_FILE_SELECTION (open_selector), home);
 
   if (gtk_dialog_run (GTK_DIALOG (open_selector)) == GTK_RESPONSE_OK) {
-    gchar *filename = (gchar*)gtk_file_selection_get_filename (GTK_FILE_SELECTION (open_selector));
+    gchar *filename = (gchar *)gtk_file_selection_get_filename (GTK_FILE_SELECTION (open_selector));
     if (filename) {
       #ifdef TSCLIENT_DEBUG
       printf ("filename: %s\n", filename);
@@ -1513,6 +1557,8 @@ on_btnProtoFile_clicked                (GtkButton       *button,
     }
   }
   gtk_widget_destroy (GTK_WIDGET (open_selector));
+
+  g_free (home);
   return;
 }
 
