@@ -29,6 +29,9 @@ main (int argc, char *argv[])
 {
   gint i;
   gchar *rdp_file_name = "";
+  gchar *home = tsc_home_path ();
+  gchar *tsc_default = g_build_path ("/", home, "default.tsc", NULL);
+  gchar *tsc_last = g_build_path ("/", home, "last.tsc", NULL);
   rdp_file *rdp = NULL;
 
   gtk_set_locale ();
@@ -52,10 +55,10 @@ main (int argc, char *argv[])
   // make sure our dirs & files exist
   tsc_check_files ();
 
-  if (g_file_test (g_build_path ("/", tsc_home_path(), "default.tsc", NULL), G_FILE_TEST_EXISTS)) {
-    rdp_file_name = g_build_path ("/", tsc_home_path(), "default.tsc", NULL);
-  } else if (g_file_test (g_build_path ("/", tsc_home_path(), "last.tsc", NULL), G_FILE_TEST_EXISTS)) {
-    rdp_file_name = g_build_path ("/", tsc_home_path(), "last.tsc", NULL);
+  if (g_file_test (tsc_default, G_FILE_TEST_EXISTS)) {
+    rdp_file_name = tsc_default;
+  } else if (g_file_test (tsc_last, G_FILE_TEST_EXISTS)) {
+    rdp_file_name = tsc_last;
   }
 
   // start at 1 because 0 contains binary name
@@ -97,9 +100,12 @@ main (int argc, char *argv[])
     rdp_file_load (rdp, rdp_file_name);
     rdp_file_set_screen (rdp, gConnect);
     g_free (rdp);
-    g_free (rdp_file_name);
   }
 
+  g_free (tsc_last);
+  g_free (tsc_default);
+  g_free (home);
+
   gtk_main ();
   return 0;
 }
