@@ -36,11 +36,13 @@ int rdp_file_init (rdp_file *rdp_in)
 
   rdp->alternate_full_switch = 0;
   rdp->alternate_shell = "";
+  rdp->attach_to_console = 0;
   rdp->audiomode = 0;
   rdp->auto_connect = 0;
   rdp->bitmapcachepersistenable = 0;
   rdp->client_hostname = "";
   rdp->compression = 0;
+  rdp->description = "";
   rdp->desktop_size_id = 0;
   rdp->desktopheight = 0;
   rdp->desktopwidth = 0;
@@ -189,11 +191,13 @@ int rdp_file_save (rdp_file *rdp_in, const char *fqpath)
   // combine the key, type, and value segments
   buffer = g_strconcat ("alternate full switch:i:", g_strdup_printf("%d",rdp->alternate_full_switch), "\r\n", NULL);
   buffer = g_strconcat (buffer, "alternate shell:s:", rdp->alternate_shell, "\r\n", NULL);
+  buffer = g_strconcat (buffer, "attach to console:i:", g_strdup_printf("%d",rdp->attach_to_console), "\r\n", NULL);
   buffer = g_strconcat (buffer, "audiomode:i:", g_strdup_printf("%d",rdp->audiomode), "\r\n", NULL);
   buffer = g_strconcat (buffer, "auto connect:i:", g_strdup_printf("%d",rdp->auto_connect), "\r\n", NULL);
   buffer = g_strconcat (buffer, "bitmapcachepersistenable:i:", g_strdup_printf("%d",rdp->bitmapcachepersistenable), "\r\n", NULL);
   buffer = g_strconcat (buffer, "client hostname:s:", rdp->client_hostname, "\r\n", NULL);
   buffer = g_strconcat (buffer, "compression:i:", g_strdup_printf("%d",rdp->compression), "\r\n", NULL);
+  buffer = g_strconcat (buffer, "description:s:", rdp->description, "\r\n", NULL);
   buffer = g_strconcat (buffer, "desktop size id:i:", g_strdup_printf("%d",rdp->desktop_size_id), "\r\n", NULL);
   buffer = g_strconcat (buffer, "desktopheight:i:", g_strdup_printf("%d",rdp->desktopheight), "\r\n", NULL);
   buffer = g_strconcat (buffer, "desktopwidth:i:", g_strdup_printf("%d",rdp->desktopwidth), "\r\n", NULL);
@@ -320,6 +324,9 @@ int rdp_file_set_screen (rdp_file *rdp_in, GtkWidget *main_window)
   case 1280:
     dsize = 4;
     break;
+  case 1400:
+    dsize = 5;
+    break;
   default:
     dsize = 1234;
     break;
@@ -455,6 +462,12 @@ int rdp_file_set_screen (rdp_file *rdp_in, GtkWidget *main_window)
   else
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
 
+  widget = lookup_widget (main_window, "chkAttachToConsole");
+  if (rdp->attach_to_console == 1)
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
+  else
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
+
   // the stuff on the resources frame
   switch (rdp->audiomode) {
   case 1:
@@ -607,6 +620,10 @@ int rdp_file_get_screen (rdp_file *rdp_in, GtkWidget *main_window)
         rdp->desktopwidth = 1280;
         rdp->desktopheight = 960;
         break;
+      case 5:
+        rdp->desktopwidth = 1400;
+        rdp->desktopheight = 1050;
+        break;
       default:
         rdp->desktopwidth = 0;
         rdp->desktopheight = 0;
@@ -723,6 +740,12 @@ int rdp_file_get_screen (rdp_file *rdp_in, GtkWidget *main_window)
   else
     rdp->hide_wm_decorations = 0;
 
+  widget = lookup_widget (main_window, "chkAttachToConsole");
+  if (gtk_toggle_button_get_active ((GtkToggleButton*)widget))
+    rdp->attach_to_console = 1;
+  else
+    rdp->attach_to_console = 0;
+
   // clean up
   g_free (value);
 
@@ -773,6 +796,9 @@ int rdp_file_set_from_line (rdp_file *rdp_in, const char *str_in)
     if (strcmp(key, "alternate shell") == 0) {
       rdp->alternate_shell = g_strdup(value);
     }
+    if (strcmp(key, "attach to console") == 0) {
+      rdp->attach_to_console = atoi(value);
+    }
     if (strcmp(key, "audiomode") == 0) {
       rdp->audiomode = atoi(value);
     }
@@ -788,6 +814,9 @@ int rdp_file_set_from_line (rdp_file *rdp_in, const char *str_in)
     if (strcmp(key, "compression") == 0) {
       rdp->compression = atoi(value);
     }
+    if (strcmp(key, "description") == 0) {
+      rdp->description = g_strdup(value);
+    }
     if (strcmp(key, "desktop size id") == 0) {
       rdp->desktop_size_id = atoi(value);
     }
@@ -824,6 +853,9 @@ int rdp_file_set_from_line (rdp_file *rdp_in, const char *str_in)
     if (strcmp(key, "full address") == 0) {
       rdp->full_address = g_strdup(value);
     }
+    if (strcmp(key, "hide wm decorations") == 0) {
+      rdp->hide_wm_decorations = atoi(value);
+    }
     if (strcmp(key, "keyboard language") == 0) {
       rdp->keyboard_language = g_strdup(value);
     }
@@ -941,6 +973,38 @@ int rdp_load_profile_launcher (GtkWidget *main_window)
 }
 
 
+/***************************************
+*                                      *
+*   read_dir_list                      *
+*                                      *
+***************************************/ 
+void read_dir_list (gchar *dir_name, GSList **list, const gchar *value) {
+  GError *error;
+  GDir *dir;
+  const gchar *name;
+  gchar *tmp_name;
+
+  dir = g_dir_open (dir_name, 0, &error);
+  while ((name = g_dir_read_name (dir)) != NULL) {
+    tmp_name = g_build_path ("/", dir_name, name, NULL);
+    if (g_file_test (tmp_name, G_FILE_TEST_IS_DIR)) {
+      read_dir_list (tmp_name, list, name);
+    } else {
+      if (g_str_has_suffix (name, ".rdp")){
+        gchar *tmp;
+        if (g_ascii_strcasecmp (value, "-1") == 0) {
+          tmp = g_strdup (name);
+        } else {
+          tmp = g_build_path ("/", value, name, NULL);
+        }
+        *list = g_slist_append (*list, tmp);
+      }
+    }
+    g_free (tmp_name);
+  }
+  g_dir_close (dir);
+}
+
 /***************************************
 *                                      *
 *   rdp_files_to_list                  *
@@ -949,40 +1013,72 @@ int rdp_load_profile_launcher (GtkWidget *main_window)
 
 int rdp_files_to_list (GSList** list)
 {
-  DIR *dptr;
   gchar *path_name;
-  struct dirent *dp;
   gint i;
-
+	
   #ifdef TSCLIENT_DEBUG
   printf ("rdp_files_to_list\n");
   #endif
 
   // create .tsclient dir in ~/
-  path_name = g_build_path ("/", tsc_home_path(), NULL);
-  if ((dptr = opendir(path_name)) == NULL) {
-    printf ("Can't open %s\n", path_name);
-    return 1;
-  }
+  path_name = tsc_home_path ();
+  
+  read_dir_list (path_name, list, "-1");
   
-  i = 0;
-  for (dp = readdir(dptr); dp != NULL; dp = readdir(dptr)) {
-    if (strlen(dp->d_name) > 4) {
-      if (dp->d_name[strlen(dp->d_name) - 3] == 'r'
-          && dp->d_name[strlen(dp->d_name) - 2] == 'd'
-          && dp->d_name[strlen(dp->d_name) - 1] == 'p') {
-        // append next structure to the list
-        *list = g_slist_append (*list, (gpointer) g_strdup_printf ("%s", dp->d_name));
-        i++;
-      }
-    }
-	}
   #ifdef TSCLIENT_DEBUG
   printf ("rdp_files_to_list count: %d\n", g_slist_length (*list));
   #endif
 
-  closedir (dptr);
-
+  g_free (path_name);
   // complete successfully
   return 0;
 }
+
+
+/***************************************
+*                                      *
+*   read_dir                           *
+*                                      *
+***************************************/ 
+void read_dir (gchar *dir_name, GHashTable *hash, const gchar *value) {
+  GError *error;
+  GDir *dir;
+  const gchar *name;
+  gchar *tmp_name;
+
+  dir = g_dir_open (dir_name, 0, &error);
+  while ((name = g_dir_read_name (dir)) != NULL) {
+    tmp_name = g_build_path ("/", dir_name, name, NULL);
+    if (g_file_test (tmp_name, G_FILE_TEST_IS_DIR)) {
+      read_dir (tmp_name, hash, name);
+    } else {
+      if (g_str_has_suffix (name, ".rdp")){
+        g_hash_table_insert (hash, g_strdup (name), g_strdup (value));
+	  }
+    }
+	g_free (tmp_name);
+  }
+  g_dir_close (dir);
+}
+
+/***************************************
+*                                      *
+*   rdp_files_to_hash                  *
+*                                      *
+***************************************/
+
+GHashTable* rdp_files_to_hash(void){
+  gchar *path_name;
+  GHashTable *hash;
+	
+  hash = g_hash_table_new (g_str_hash, g_str_equal);
+	
+  #ifdef TSCLIENT_DEBUG
+  printf ("rdp_files_to_list\n");
+  #endif
+
+  path_name = g_build_path ("/", g_get_home_dir (), ".tsclient", NULL);
+  read_dir (path_name, hash, "-1");
+  g_free (path_name);
+  return hash;
+};
