@@ -1,4 +1,3 @@
-
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -23,9 +22,23 @@
 #include <dirent.h> 
 #include <fcntl.h> 
 
+#include <arpa/inet.h>
+#ifdef HAVE_SYS_SOCKIO_H
+#include <sys/sockio.h>
+#endif
+#include <errno.h>
+#include <signal.h>
+
 #include "rdpfile.h"
 #include "support.h"
 
+/* Stolen from jirka's vicious lib */
+#define VE_IGNORE_EINTR(expr)                   \
+        do {                                    \
+                errno = 0;                      \
+                expr;                           \
+        } while G_UNLIKELY (errno == EINTR);
+
 
 GtkWidget* lookup_widget (GtkWidget *widget, const gchar *widget_name) {
   GtkWidget *parent, *found_widget;
@@ -144,7 +157,8 @@ GtkWidget *tsc_option_menu_factory (gchar**vmnemonic) {
 ***************************************/
  
 int tsc_check_files () {
-  const gchar *file_name = (const gchar *)g_build_path ("/", tsc_home_path(), "mru.tsc", NULL);
+  gchar *home = tsc_home_path ();
+  gchar *file_name = g_build_path ("/", home, "mru.tsc", NULL);
   int ret = 0;
 
   #ifdef TSCLIENT_DEBUG
@@ -152,7 +166,7 @@ int tsc_check_files () {
   #endif
 
   // create .tsclient dir in ~/
-  mkdir (tsc_home_path(), 0700);
+  mkdir (home, 0700);
 
   // create mru.tsc dir in ~/.tsclient/
   if (!g_file_test (file_name, G_FILE_TEST_EXISTS)) {
@@ -166,6 +180,9 @@ int tsc_check_files () {
     if (ret > 0) close (ret);
   }
 
+  /* clean up */
+  g_free (file_name);
+  g_free (home);
   /* complete successfully */
   return 0;
 }
@@ -177,7 +194,7 @@ int tsc_check_files () {
 *                                      *
 ***************************************/
  
-const gchar *tsc_home_path () {
+gchar *tsc_home_path () {
 
   #ifdef TSCLIENT_DEBUG
   printf ("tsc_home_path\n");
@@ -188,6 +205,124 @@ const gchar *tsc_home_path () {
 }
 
 
+/**
+ * tsc_get_free_display
+ * @start: Start at this display, use 0 as safe value.
+ * @server_uid: UID of X server
+ *
+ * Evil function to figure out which display number is free.
+ *
+ * Stolen from gdm-2.6.0:daemon/misc.c
+ *
+ * Returns: A free X display number or -1 on failure.
+ */
+
+int tsc_get_free_display (int start, uid_t server_uid)
+{
+  int sock;
+  int i;
+  struct sockaddr_in serv_addr = {0};
+
+  serv_addr.sin_family = AF_INET;
+  serv_addr.sin_addr.s_addr = htonl (INADDR_LOOPBACK);
+
+  /* Cap this at 3000, I'm not sure we can ever seriously
+   * go that far */
+  for (i = start; i < 3000; i ++) {
+    struct stat s;
+    char buf[256];
+    FILE *fp;
+    int r;
+
+#ifdef ENABLE_IPV6
+    if (have_ipv6 ()) {
+      struct sockaddr_in6 serv6_addr= {0};
+
+      sock = socket (AF_INET6, SOCK_STREAM,0);
+
+      serv6_addr.sin6_family = AF_INET6;
+      serv6_addr.sin6_addr = in6addr_loopback;
+      serv6_addr.sin6_port = htons (6000 + i);
+      errno = 0;
+      VE_IGNORE_EINTR (connect (sock,
+                                (struct sockaddr *)&serv6_addr,
+                                sizeof (serv6_addr)));
+    }
+    else
+#endif
+      {
+        sock = socket (AF_INET, SOCK_STREAM, 0);
+
+        serv_addr.sin_port = htons (6000 + i);
+
+        errno = 0;
+        VE_IGNORE_EINTR (connect (sock,
+                                  (struct sockaddr *)&serv_addr,
+                                  sizeof (serv_addr)));
+      }
+    if (errno != 0 && errno != ECONNREFUSED) {
+      VE_IGNORE_EINTR (close (sock));
+      continue;
+    }
+    VE_IGNORE_EINTR (close (sock));
+
+    /* if lock file exists and the process exists */
+    g_snprintf (buf, sizeof (buf), "/tmp/.X%d-lock", i);
+    VE_IGNORE_EINTR (r = stat (buf, &s));
+    if (r == 0 &&
+        ! S_ISREG (s.st_mode)) {
+      /* Eeeek! not a regular file?  Perhaps someone
+         is trying to play tricks on us */
+      continue;
+    }
+    VE_IGNORE_EINTR (fp = fopen (buf, "r"));
+    if (fp != NULL) {
+      char buf2[100];
+      char *getsret;
+      VE_IGNORE_EINTR (getsret = fgets (buf2, sizeof (buf2), fp));
+      if (getsret != NULL) {
+        gulong pid;
+        if (sscanf (buf2, "%lu", &pid) == 1 &&
+            kill (pid, 0) == 0) {
+          VE_IGNORE_EINTR (fclose (fp));
+          continue;
+        }
+
+      }
+      VE_IGNORE_EINTR (fclose (fp));
+
+      /* whack the file, it's a stale lock file */
+      VE_IGNORE_EINTR (unlink (buf));
+    }
+
+    /* if starting as root, we'll be able to overwrite any
+     * stale sockets or lock files, but a user may not be
+     * able to */
+    if (server_uid > 0) {
+      g_snprintf (buf, sizeof (buf),
+                  "/tmp/.X11-unix/X%d", i);
+      VE_IGNORE_EINTR (r = stat (buf, &s));
+      if (r == 0 &&
+          s.st_uid != server_uid) {
+        continue;
+      }
+
+      g_snprintf (buf, sizeof (buf),
+                  "/tmp/.X%d-lock", i);
+      VE_IGNORE_EINTR (r = stat (buf, &s));
+      if (r == 0 &&
+          s.st_uid != server_uid) {
+        continue;
+      }
+    }
+
+    return i;
+  }
+
+  return -1;
+}
+
+
 /***************************************
 *                                      *
 *   tsc_launch_remote                  *
@@ -280,6 +415,10 @@ int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
           sprintf(buffer, "-g1280x960");
           c_argv[c_argc++] = g_strdup (buffer);
           break;
+        case 1400:
+          sprintf(buffer, "-g1400x1050");
+          c_argv[c_argc++] = g_strdup (buffer);
+          break;
         default:
           break;
         }
@@ -332,6 +471,11 @@ int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
         c_argv[c_argc++] = g_strdup (buffer);
       }
 
+      if (rdp->attach_to_console == 1) {
+        sprintf ( buffer, "-0");
+        c_argv[c_argc++] = g_strdup (buffer);
+      }
+
       if ( rdp->keyboard_language && strlen (rdp->keyboard_language) ) {
         sprintf ( buffer, "-k");
         c_argv[c_argc++] = g_strdup (buffer);
@@ -422,27 +566,30 @@ int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
         }
       }
 
+      /*  this is diff in all vnc vers
       switch (rdp->session_bpp) {
       case 8:
-        sprintf(buffer, "-d");
+        sprintf(buffer, "-bgr233");
+        c_argv[c_argc++] = g_strdup (buffer);
+        sprintf(buffer, "-depth");
         c_argv[c_argc++] = g_strdup (buffer);
         sprintf(buffer, "8");
         c_argv[c_argc++] = g_strdup (buffer);
         break;
       case 15:
-        sprintf(buffer, "-d");
+        sprintf(buffer, "-depth");
         c_argv[c_argc++] = g_strdup (buffer);
         sprintf(buffer, "15");
         c_argv[c_argc++] = g_strdup (buffer);
         break;
       case 16:
-        sprintf(buffer, "-d");
+        sprintf(buffer, "-depth");
         c_argv[c_argc++] = g_strdup (buffer);
         sprintf(buffer, "16");
         c_argv[c_argc++] = g_strdup (buffer);
         break;
       case 24:
-        sprintf(buffer, "-d");
+        sprintf(buffer, "-depth");
         c_argv[c_argc++] = g_strdup (buffer);
         sprintf(buffer, "24");
         c_argv[c_argc++] = g_strdup (buffer);
@@ -450,6 +597,7 @@ int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
       default:
         break;
       }
+      */
 
       if (rdp->no_motion_events == 1) {
         sprintf(buffer, "-v");
@@ -469,6 +617,7 @@ int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
       c_argv[c_argc++] = g_strdup (buffer);
 
     } else if (rdp->protocol == 2) {
+      int display;
     
       if (g_find_program_in_path ("Xnest")) {
         sflags += G_SPAWN_SEARCH_PATH;
@@ -480,7 +629,13 @@ int tsc_launch_remote (rdp_file *rdp_in, int launch_async)
         return 1;
       }
         
-      sprintf(buffer, ":1");
+      /* Starting search from :1 (assuming we run at :0) */
+      display = tsc_get_free_display (1, getuid());
+      if (-1 == display) {
+        tsc_error_message (_("Could not find a free X display."));
+        return 1;
+      }
+      sprintf(buffer, ":%d", display);
       c_argv[c_argc++] = strdup(buffer);
 
       sprintf(buffer, "-once");
@@ -740,7 +895,7 @@ void tsc_about_dialog ()
   const gchar *name      = _("Terminal Server Client");
   const gchar *version   = VERSION;
   const gchar *copyright = "(C) 2003 Erick Woods\n<http://www.gnomepro.com/tsclient/>";
-  const gchar *authors[] = { "Erick Woods <erick@gnomepro.com>", NULL };
+  const gchar *authors[] = { "Erick Woods <erick@gnomepro.com>", "Kyle Davis <kyle@gnomeuser.org>", NULL };
   const gchar *documenters[] = { NULL };
   const gchar *translator_credits = NULL;
   GdkPixbuf *pixbuf = create_pixbuf ("tsclient.png");
@@ -778,6 +933,7 @@ void tsc_about_dialog ()
 void tsc_quick_pick_activate (GtkMenuItem *menuitem, gpointer user_data)
 {
   gchar *file_name;
+  gchar *home = tsc_home_path ();
   rdp_file *rdp = NULL;
 
   #ifdef TSCLIENT_DEBUG
@@ -785,7 +941,7 @@ void tsc_quick_pick_activate (GtkMenuItem *menuitem, gpointer user_data)
   #endif
 
   // build path to file
-  file_name = g_build_path ("/", tsc_home_path(), g_strdup ((const gchar*)user_data), NULL);
+  file_name = g_build_path ("/", home, (const gchar *)user_data, NULL);
   // check for file in ~/
   if (g_file_test (file_name, G_FILE_TEST_EXISTS)) {
     // exists, load it
@@ -796,8 +952,9 @@ void tsc_quick_pick_activate (GtkMenuItem *menuitem, gpointer user_data)
       //mru_add_server (rdp->full_address);
     }
   }
-  g_free (file_name);
   g_free (rdp);
+  g_free (file_name);
+  g_free (home);
   return;
 }
 
@@ -827,17 +984,20 @@ tsc_set_protocol_widgets (GtkWidget *main_win, gint protocol)
   switch (protocol) {
   case 0:  // rdp v4
   case 4:  // rdp v5
+    gtk_widget_set_sensitive ((GtkWidget*) g_object_get_data (G_OBJECT (main_win), "txtPassword"), TRUE);
     gtk_widget_set_sensitive ((GtkWidget*) g_object_get_data (G_OBJECT (main_win), "btnProtoFile"), FALSE);
     gtk_widget_set_sensitive ((GtkWidget*) g_object_get_data (G_OBJECT (main_win), "txtProtoFile"), FALSE);
     gtk_widget_set_sensitive ((GtkWidget*) g_object_get_data (G_OBJECT (main_win), "chkHideWMDecorations"), TRUE);
     break;
   case 1:  // vnc
+    gtk_widget_set_sensitive ((GtkWidget*) g_object_get_data (G_OBJECT (main_win), "txtPassword"), FALSE);
     gtk_widget_set_sensitive ((GtkWidget*) g_object_get_data (G_OBJECT (main_win), "vbxSound"), FALSE);
     gtk_widget_set_sensitive ((GtkWidget*) g_object_get_data (G_OBJECT (main_win), "chkStartProgram"), FALSE);
     gtk_widget_set_sensitive ((GtkWidget*) g_object_get_data (G_OBJECT (main_win), "txtProgramPath"), FALSE);
     gtk_widget_set_sensitive ((GtkWidget*) g_object_get_data (G_OBJECT (main_win), "txtStartFolder"), FALSE);
     break;
   case 2:  // xdmcp
+    gtk_widget_set_sensitive ((GtkWidget*) g_object_get_data (G_OBJECT (main_win), "txtPassword"), FALSE);
     gtk_widget_set_sensitive ((GtkWidget*) g_object_get_data (G_OBJECT (main_win), "btnProtoFile"), FALSE);
     gtk_widget_set_sensitive ((GtkWidget*) g_object_get_data (G_OBJECT (main_win), "txtProtoFile"), FALSE);
 
@@ -848,6 +1008,7 @@ tsc_set_protocol_widgets (GtkWidget *main_win, gint protocol)
     gtk_widget_set_sensitive ((GtkWidget*) g_object_get_data (G_OBJECT (main_win), "txtStartFolder"), FALSE);
     break;
   case 3:  // ica
+    gtk_widget_set_sensitive ((GtkWidget*) g_object_get_data (G_OBJECT (main_win), "txtPassword"), TRUE);
     break;
   default:
     break;
